﻿local dewdrop = AceLibrary('Dewdrop-2.0')
local L = AceLibrary('AceLocale-2.2'):new('ClassTimer')
local sm = AceLibrary('SharedMedia-1.0')

--Ace2 Registering
ClassTimer = AceLibrary('AceAddon-2.0'):new('AceConsole-2.0', 'AceEvent-2.0', 'AceDB-2.0')

ClassTimer:RegisterDB('ClassTimerDB')
 
function ClassTimer:OnInitialize()
--Revision 
self.revision = tonumber(string.match('$Rev: 36832 $', '%d+'))
self.version = self.revision and self.version .. ' |cffff8888r'..self.revision..'|r'
self:RulesInit()
self:RegisterDefaults('profile', self.defaults)
self:RegisterChatCommand('/ClassTimerConsole', self.Menu)
dewdrop:Register('ClassTimer', 'children', function() dewdrop:FeedAceOptionsTable(self.Menu) end)
self:RegisterChatCommand('/ClassTimer', '/ct', function() dewdrop:Open('ClassTimer') end )
end

function ClassTimer:OnEnable()
	self:RulesSetup()
end

do
	local function dragstart()
		this:StartMoving()
	end
	local function dragstop()
		ClassTimer.db.profile.Units[this.unit].x = this:GetLeft()
		ClassTimer.db.profile.Units[this.unit].y = this:GetBottom()
		this:StopMovingOrSizing()
	end
	local function set(table, value)
		local s1, s2 = unpack(table)
		if s2 then
			ClassTimer.db.profile.Units[s1][s2] = value
		else
			ClassTimer.db.profile[s1] = value
		end
			ClassTimer.ApplySettings()
	end
	local function get(table)
		local s1, s2 = unpack(table)
		if s2 then
			return ClassTimer.db.profile.Units[s1][s2]
		else
			return ClassTimer.db.profile[s1]
		end
	end
	local function setcolor(field, ...)
		ClassTimer.db.profile[field] = {...}
		ClassTimer.ApplySettings()
	end
	local function getcolor(field)
		return unpack(ClassTimer.db.profile[field])
	end 
	local function hideoptions(unit)	
		return not ClassTimer.db.profile.Units[unit[1]].enable
	end
	local function Unit(unit)
		return {
			type = 'group',
			name = L[unit],
			desc = L[unit],
			order = 1,
			disabled = function() return unit ~= 'player' and ClassTimer.db.profile.AllInOne end,
			args = {
				show = {
					type = 'toggle',
					name = L['Enable %s']:format(L[unit]),
					desc = L['Show buffs/debuffs for your %s']:format(L[unit]),
					get = get,
					set = set,
					passValue = {unit, 'enable'},
					order = 1,
				},
				space = {
					type = 'header',
					order = 2,
					passValue = {unit},
					hidden = hideoptions,
				},
				buffs = {
					type = 'toggle',
					name = L['Enable Buffs'],
					desc = L['Show buffs for your %s']:format(L[unit]),
					get = get,
					set = set,
					order = 3,
					passValue = {unit, 'buffs'},
					hidden = hideoptions,
				},
				debuffs = {
					type = 'toggle',
					name = L['Enable Debuffs'],
					desc = L['Show debuffs for your %s']:format(L[unit]),
					get = get,
					order = 4,
					set = set,
					passValue = {unit, 'debuffs'},
					hidden = hideoptions,
				},
				lock = {
					type = 'toggle',
					name = L['Lock'],
					desc = L['Toggle Cast Bar lock'],
					order = 5,
					hidden = hideoptions,
					passValue = {unit},
					get = function()
						return ClassTimer.locked[unit]
					end,
					set = function(_, v)
						if v then
							ClassTimer.bars[unit][1]:EnableMouse(false)
							ClassTimer.bars[unit][1]:SetScript('OnDragStart', nil)
							ClassTimer.bars[unit][1]:SetScript('OnDragStop', nil)
						else
							local bar = ClassTimer.bars[unit][1]
							bar.icon:SetTexture(ClassTimer.hasIcon)
							bar.text:SetText(L['Drag to move'])
							bar.startTime = GetTime()
							bar.endTime = GetTime() + 120
							bar:SetMinMaxValues(GetTime(),  GetTime() + 120)
							bar:RegisterForDrag('LeftButton')
							bar.unit = unit
							bar:EnableMouse(true)
							bar:SetScript('OnDragStart', dragstart)
							bar:SetScript('OnDragStop', dragstop)
							bar:SetAlpha(1)
							bar:Show()
						end
						ClassTimer.locked[unit] = v
						ClassTimer:UpdateUnitBars(unit)
					end,
				},
				scale = {
					type = 'range',
					name = L['Scale'],
					desc = L['Set the scale of the bars for your %s']:format(L[unit]),
					get = get,
					set = set,
					min = .1,
					max = 2,
					step = .01,
					bigStep = .1,
					order = 12,
					passValue = {unit, 'scale'},
					hidden = hideoptions,
				},
				spacing = {
					type = 'range',
					name = L['Spacing'],
					desc = L['Tweak the space between bars for your %s']:format(L[unit]),
					min = -5,
					max = 5,
					step = .1,
					bigStep = 1,
					get = get,
					set = set,
					passValue = {unit, 'spacing'},
					order = 13,
					hidden = hideoptions,
				},
				iconside = {
					type = 'text',
					name = L['Icon Position'],
					desc = L['Set the side of the bar that the icon appears on'],
					get = get,
					set = set,
					validate = {L['Left'], L['Right']},
					passValue = {unit, 'iconSide'},
					order = 20,
					hidden = hideoptions,
				},
				showicons = {
					type = 'toggle',
					name = L['Show Icons'],
					desc = L['Show icons on buffs and debuffs for your %s']:format(L[unit]),
					get = get,
					set = set,
					passValue = {unit, 'icons'},
					order = 30,
					hidden = hideoptions,
				},
				showiconsOnly = {
					type = 'toggle',
					name = L['Show Only Icons'],
					desc = L['Show only icons and timeleft for your %s']:format(L[unit]),
					get = get,
					set = set,
					passValue = {unit, 'showIcons'},
					order = 31,
					hidden = hideoptions,
				},
				growUp = {
					type = 'toggle',
					name = L['Grow Up'],
					desc = L['Set %s bars to grow up']:format(L[unit]),
					get = get,
					set = set,
					order = 32,
					passValue = {unit, 'growup'},
					hidden = hideoptions,
				},
			}
		}
	end

	ClassTimer.Menu = { 
	    type ='group',
	    args = {
			spacer = {
				type = 'header',
				order = 3,
			},
			BarSettings = {
				type = 'group',
				name = L['Bar Settings'],
				desc = L['Bar Settings'],
				order = 3,
				disabled = '~IsActive',
				args = {
					focus = Unit('focus'),
					target = Unit('target'),
					player = Unit('player'),
					pet = Unit('pet'),
					alpha = {
						type = 'range',
						name = L['Alpha'],
						desc = L['Set the alpha of the bars'],
						min = 0.05,
						max = 1,
						step = 0.01,
						bigStep = .1,
						isPercent = true,
						get = get,
						set = set,
						passValue = {'alpha'},
						order = 20,
					},
					fontsize = {
						type = 'range',
						name = L['Font Size'],
						desc = L['Set the font size for the bars'],
						min = 3,
						max = 15,
						step = 2,
						bigStep = .05,
						get = get,
						set = set,
						passValue = {'fontsize'},
						order = 21,
					},
					height = {
						type = 'range',
						name = L['Bar Height'],
						desc = L['Set the height of the bars'],
						min = 4,
						max = 25,
						step = 2,
						bigStep = 5,
						get = get,
						set = set,
						passValue = {'height'},
						order = 22,
					},
					width = {
						type = 'range',
						name = L['Bar Width'],
						desc = L['Set the width of the bars'],
						min = 50,
						max = 300,
						step = 2,
						bigStep = 5,
						get = get,
						set = set,
						passValue = {'width'},
						order = 23,
					},
					font = {
						type = 'text',
						name = L['Font'],
						desc = L['Set the font used in the bars'],
						validate = sm:List('font'),
						get = get,
						set = set,
						passValue = {'font'},
						order = 40,
					},
					texture = {
						type = 'text',
						name = L['Texture'],
						desc = L['Set the bar Texture'],
						validate = sm:List('statusbar'),
						order = 41,
						get = get,
						set = set,
						passValue = {'texture'},
					},
					text = {
						type = 'text',
						name = L['Bar Text'],
						desc = L['Set the bar text'],
						get = get,
						set = set,
						order = 42,
						usage = L['<%s for spell, %a for applications, %n for name, %u for unit. %s must be included>'],
						validate = function(name) return string.find(name, '%%s') end,
						passValue = {'bartext'},
					},
					textcolor = {
						type = 'color',
						name = L['Text Color'],
						desc = L['Set the color of the text for the bars'],
						get = getcolor,
						set = setcolor,
						passValue = 'textcolor',
						order = 60,
					},
					buffcolor = {
						type = 'color',
						name = L['Buff Color'],
						desc = L['Set the color of the bars for buffs'],
						get = getcolor,
						set = setcolor,
						hasAlpha = true,
						passValue = 'buffcolor',
						order = 61,
					},
					debuffcolor = {
						type = 'color',
						name = L['Debuff Color'],
						desc = L['Set the color of the bars for debuffs'],
						get = getcolor,
						set = setcolor,
						hasAlpha = true,
						passValue = 'debuffcolor',
						order = 62,
					},
					backgroundcolor = {
						type = 'color',
						name = L['Background Color'],
						desc = L['Set the color of the bars background'],
						get = getcolor,
						set = setcolor,
						hasAlpha = true,
						passValue = 'backgroundcolor',
						order = 62,
					},
					AllInOne = {
						type = 'toggle',
						name = L['AllInOne'],
						desc = L['Display all the buffs and debuffs on the "player" bar'],
						get = get,
						set = set,
						passValue = {'AllInOne'},
						order = 80,
					},
					nametext = {
						type = 'toggle',
						name = L['Name Text'],
						desc = L['Display the names of buffs/debuffs on their bars'],
						get = get,
						set = set,
						passValue = {'nametext'},
						order = 80,
					},
					timetext = {
						type = 'toggle',
						name = L['Time Text'],
						desc = L['Display the time remaining on buffs/debuffs on their bars'],
						get = get,
						set = set,
						passValue = {'timetext'},
						order = 81,
					},
				},
			}
		}
	}
end
